using AzureBakery.Production.ManagementApplication.Services;
using AzureBakery.Production.ManagementApplication.Services.Interfaces;
using AzureBakery.Production.Model;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using System.Linq;

namespace AzureBakery.Production.ManagementApplication.ViewModel
{
    /// <summary>
    /// This class contains properties that the main View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm
    /// </para>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {
        private IDataService _dataService = null;
        private IManagementService _managementService = null;

        private readonly ObservableCollection<OrderViewModel> _orders = null;
        private readonly ObservableCollection<BatchViewModel> _batches = null;

        private RelayCommand<OrderViewModel> _dispatchOrderCommand = null;
        private RelayCommand<BatchViewModel> _processingBatchCommand = null;
        private RelayCommand<BatchViewModel> _completeBatchCommand = null;

        public ObservableCollection<OrderViewModel> Orders
        {
            get { return this._orders; }
        }

        public ObservableCollection<BatchViewModel> Batches
        {
            get { return this._batches; }
        }

        public RelayCommand<OrderViewModel> DispatchOrderCommand
        {
            get { return this._dispatchOrderCommand; }
        }

        public RelayCommand<BatchViewModel> ProcessingBatchCommand
        {
            get { return this._processingBatchCommand; }
        }

        public RelayCommand<BatchViewModel> CompleteBatchCommand
        {
            get { return this._completeBatchCommand; }
        }

        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel(IDataService dataService, IManagementService managementService)
        {
            if (!ViewModelBase.IsInDesignModeStatic)
            {
                this._dataService = dataService;
                this._managementService = managementService;

                this._orders = new ObservableCollection<OrderViewModel>();
                this._batches = new ObservableCollection<BatchViewModel>();

                this.CreateCommands();

                this.RegisterManagementService();

                this.InitializeData();
            }
        }

        private void CreateCommands()
        {
            this._dispatchOrderCommand = new RelayCommand<OrderViewModel>((o) => 
            {
                var order = o.Order.Clone();
                order.Status = OrderStatus.ReadyForDispatch;

                this._managementService.UpdateOrder(order);
            });

            this._processingBatchCommand = new RelayCommand<BatchViewModel>((b) =>
            {
                var batch = b.Batch.Clone();
                batch.Status = BatchStatus.Processing;
                this._managementService.UpdateBatch(batch);
            });

            this._completeBatchCommand = new RelayCommand<BatchViewModel>((b) =>
            {
                var batch = b.Batch.Clone();
                batch.Status = BatchStatus.Complete;
                this._managementService.UpdateBatch(batch);
            });
        }

        private async void RegisterManagementService()
        {
            await this._managementService.Register();
            this._managementService.OrderUpdated += _managementService_OrderUpdated;
            this._managementService.BatchUpdated += _managementService_BatchUpdated;
        }

        private void _managementService_BatchUpdated(object sender, Batch e)
        {
            // Get local batch and update
            var batch = this._batches.SingleOrDefault(b => b.Id == e.Id);

            if (batch != null)
                batch.Status = e.Status;
        }

        private void _managementService_OrderUpdated(object sender, Order e)
        {
            // Get local order and update
            var order = this._orders.SingleOrDefault(o => o.Id == e.Id);

            if (order != null)
                order.Status = e.Status;
        }

        private async void InitializeData()
        {
            // Get orders
            var orders = await this._dataService.GetOrders();
            foreach(var o in orders)
                this._orders.Add(new OrderViewModel(o));

            // Get batches
            var batches = await this._dataService.GetBatches();
            foreach (var b in batches)
                this._batches.Add(new BatchViewModel(b));
        }
    }
}