using AzureBakery.Sales.Model;
using AzureBakery.Sales.OrderSim.Services.Intefaces;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;

namespace AzureBakery.Sales.OrderSim.ViewModel
{
    /// <summary>
    /// This class contains properties that the main View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm
    /// </para>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {
        private IEnumerable<ProductViewModel> _products = null;
        private IEnumerable<int> _rates = new List<int> { 0, 5, 10, 15, 20, 25, 30, 50, 100, 150, 200, 250, 300, 500, 1000, 3000, 5000, 10000, 30000, 60000 };
        private IEnumerable<int> _threads = new List<int> { 1, 2, 3, 4, 8, 16 };
        private int _rate = 1000;
        private int _thread = 1;
        private int _ordersSent = 0;
        private int _ordersPurged = 0;
        private bool _canStart = true;
        private bool _canStop = false;
        private bool _canPurge = true;
        private RelayCommand _startCommand = null;
        private RelayCommand _stopCommand = null;
        private RelayCommand _purgeCommand = null;
        private CancellationTokenSource _cancellationTokenSource;

        public string Title { get { return "Order Message Simulator"; } }

        public IEnumerable<int> Rates { get { return this._rates; } }

        public IEnumerable<int> Threads { get { return this._threads; } } 

        private readonly IDataService _dataService = null;
        private readonly IMessagingService _messagingService = null;

        public IEnumerable<ProductViewModel> Products
        {
            get { return this._products; }
            set
            {
                if(this._products != value)
                {
                    this._products = value;

                    base.RaisePropertyChanged(() => this.Products);
                }
            }
        }

        public int Rate
        {
            get { return this._rate; }
            set
            {
                if (this._rate != value)
                {
                    this._rate = value;

                    base.RaisePropertyChanged(() => this.Rate);
                }
            }
        }

        public int Thread
        {
            get { return this._thread; }
            set
            {
                if (this._thread != value)
                {
                    this._thread = value;

                    base.RaisePropertyChanged(() => this.Thread);
                }
            }
        }

        public int OrdersSent
        {
            get { return this._ordersSent; }
            set
            {
                if (this._ordersSent != value)
                {
                    this._ordersSent = value;

                    base.RaisePropertyChanged(() => this.OrdersSent);
                }
            }
        }

        public int OrdersPurged
        {
            get { return this._ordersPurged; }
            set
            {
                if (this._ordersPurged != value)
                {
                    this._ordersPurged = value;

                    base.RaisePropertyChanged(() => this.OrdersPurged);
                }
            }
        }

        public RelayCommand StartCommand { get { return this._startCommand; } }

        public RelayCommand StopCommand { get { return this._stopCommand; } }

        public RelayCommand PurgeCommand { get { return this._purgeCommand; } }

        public bool CanStart
        {
            get { return this._canStart; }
            set
            {
                if (this._canStart != value)
                {
                    this._canStart = value;

                    base.RaisePropertyChanged(() => this.CanStart);
                }
            }
        }

        public bool CanStop
        {
            get { return this._canStop; }
            set
            {
                if (this._canStop != value)
                {
                    this._canStop = value;

                    base.RaisePropertyChanged(() => this.CanStop);
                }
            }
        }

        public bool CanPurge
        {
            get { return this._canPurge; }
            set
            {
                if (this._canPurge != value)
                {
                    this._canPurge = value;

                    base.RaisePropertyChanged(() => this.CanPurge);
                }
            }
        }

        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel(IDataService dataService, IMessagingService messagingService)
        {
            this._dataService = dataService;
            this._messagingService = messagingService;

            this._startCommand = new RelayCommand(this.Start, () => this.CanStart);
            this._stopCommand = new RelayCommand(this.Stop, () => this.CanStop);
            this._purgeCommand = new RelayCommand(this.Purge, () => this.CanPurge);

            if (IsInDesignMode)
            {
                // Code runs in Blend --> create design time data.
            }
            else
            {
                // Code runs "for real"
                this.Initialise();
                this._messagingService.CreateSubscriptionAsync();
            }
        }

        private void Initialise()
        {
            this.Products = this._dataService
                .GetProducts()
                .Select(p => new ProductViewModel() { Id = p.Id, Name = p.Name, Product = p })
                .ToList();

            this.CanStart = true;
        }               

        private void Start()
        {
            this._cancellationTokenSource = new CancellationTokenSource();

            this.CanStart = false;
            this.CanStop = true;

            // Set maximum number of connections to match thread count
            ServicePointManager.DefaultConnectionLimit = this._thread;

            for (int i = 0; i < this._thread; i++ )
            {
                var thread = new Thread(new ThreadStart(this.DoWork));
                thread.Start();
            }
        }

        private void Stop()
        {
            this._cancellationTokenSource.Cancel();
        }

        private async void Purge()
        {
            this.CanPurge = false;

            this.OrdersPurged += await this._messagingService.PurgeSubscriptionAsync();

            this.CanPurge = true;
        }

        private async void DoWork()
        {
            while(!this._cancellationTokenSource.IsCancellationRequested)
            {
                var order = new Order();

                var rnd = new Random();
                
                // Add items to order
                foreach(var product in this.Products)
                {
                    // 3/4 chance of adding product
                    if (rnd.Next(0, 3) < 1)
                        continue;

                    // Random  quantity between min and max
                    var qty = rnd.Next(product.Min, product.Max);

                    if (qty == 0)
                        continue;

                    product.Sent += qty;

                    // add item to order
                    order.Items.Add(new OrderItem() { Product = product.Product, Quantity = qty });
                }

                // Send order
                await this._messagingService.SendOrderAsync(order);

                this.OrdersSent++;

                // Wait for next scan
                try
                {
                    await Task.Delay(this._rate, this._cancellationTokenSource.Token);
                }
                catch(TaskCanceledException)
                {

                }
            }

            this.CanStart = true;
            this.CanStop = false;
        }
    }
}