﻿using System;
using System.ServiceModel.PeerResolvers;
using AzureBakery.Sales.Model;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Table;
using OrderProcessorRole.Labels;
using Supply.Model;
using System.Net;
using System.Threading.Tasks;

namespace OrderProcessorRole.Business
{
    public class OrderProcessor
    {
        private CloudStorageAccount _storageAccount;
        private CloudTable _table;
        private CloudBlobContainer _container;

        public OrderProcessor()
        {
            this.Initialize();
        }

        private async void Initialize()
        {
            // Retrieve the storage account from the connection string.
            this._storageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("StorageConnectionString"));
            await this.CreateTable();
            await this.CreateContainer();
        }

        private async Task CreateTable()
        {
            var tableClient = _storageAccount.CreateCloudTableClient();

            this._table = tableClient.GetTableReference("orders");
            await this._table.CreateIfNotExistsAsync();
        }

        private async Task CreateContainer()
        {
            var blobClient = this._storageAccount.CreateCloudBlobClient();

            this._container = blobClient.GetContainerReference("barcodes");
            await this._container.CreateIfNotExistsAsync();
        }

        public async Task Process(Order order)
        {
            await this.UpdateTable(order);
            await this.CreateLabel(order);
        }

        private async Task UpdateTable(Order order)
        {
            // Check if order exists
            var retrieveOp = TableOperation.Retrieve<OrderEntity>(order.Reference, order.Reference);
            var retieveResult = (await this._table.ExecuteAsync(retrieveOp));

            var entity = (OrderEntity)retieveResult.Result;

            if (entity != null && entity.Status != order.Status.ToString())
            {
                entity.Status = order.Status.ToString();
            }
            else if (entity == null)
            {
                entity = new OrderEntity(order);
            }
            else
            {
                return;
            }

            // Update entity
            var updateOp = TableOperation.InsertOrReplace(entity);
            var updateResult = await this._table.ExecuteAsync(updateOp);
        }

        private async Task CreateLabel(Order order)
        {
            try
            {
                // Get blob ref and check if it exists   
                var blobRef = this._container.GetBlockBlobReference(order.Reference);
                if (await blobRef.ExistsAsync())
                    return;

                // Get label data
                var data = await new LabelGenerator().GeBytes(order.Customer);

                // Upload to a blob
                await blobRef.UploadFromByteArrayAsync(data, 0, data.Length);
            }
            catch (Exception)
            {

                throw;
            }


        }
    }
}
