﻿using System;
using System.Diagnostics;
using System.Threading.Tasks;
using Microsoft.ServiceBus.Messaging;
using AzureBakery.Sales.Model;

namespace AzureBakery.Sales.CustomerWebsite.Services
{
    public class MessagingService
    {
        private readonly TopicClient _topicClient = null;
        private readonly string _topicName = "Order";

        public MessagingService()
        {
            // Create TopicClient (this uses the Microsoft.ServiceBus.ConnectionString app setting
            this._topicClient = TopicClient.Create(_topicName);
        }

        public async Task<bool> SendOrderAsync(Order order, int retires = 0)
        {
            Trace.TraceInformation("MessagingService SendOrder - Start");

            // Create message with Order body
            BrokeredMessage message = new BrokeredMessage(order);

            // Set status to New
            message.Properties["Status"] = "New";

            // Unique Id
            message.MessageId = Guid.NewGuid().ToString();

            var sent = await this.SendMessageAsync(message, retires);

            Trace.TraceInformation("MessagingService SendOrder - Start");

            return sent;
        }

        private async Task<bool> SendMessageAsync(BrokeredMessage message, int retires = 0)
        {
            Trace.TraceInformation("MessagingService SendMessage - Start");

            int retry = 0;
            bool sent = false;

            while (true)
            {
                try
                {
                    // Send message
                    await this._topicClient.SendAsync(message);
                    sent = true;
                    break;
                }
                catch (MessagingException ex)
                {
                    // If an exception is transient we will retry
                    if (ex.IsTransient)
                    {
                        retry++;

                        Trace.TraceInformation("MessagingService SendMessage - Retry: {0}", retry);
                    }
                    else
                    {
                        Trace.TraceError("MessagingService SendOrder Error: {0}", ex);

                        throw;
                    }
                }

                if (retry > retires)
                    break;

                // Back-off for retry
                await Task.Delay(2000);
            }

            Trace.TraceInformation("MessagingService SendMessage - End");

            return sent;
        }
    }
}