﻿using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;
using System;
using System.Net;
using System.Threading.Tasks;

namespace OrderProcessorRole.Messaging
{
    public abstract class TopicProcessorBase : IDisposable
    {
        private SubscriptionClient _subscriptionClient = null;
        private string _topicName = null;
        private string _subscriptionName = null;
        private string _sqlFilter = null;

        public bool IsRunning { get; private set; }

        public TopicProcessorBase(string topicName, string subscriptionName, string sqlFilter)
        {
            this._topicName = topicName;
            this._subscriptionName = subscriptionName;
            this._sqlFilter = sqlFilter;

            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            // Create subscription
            this.CreateSubscription();

            // Create subscription client
            this._subscriptionClient = SubscriptionClient.Create(_topicName, _subscriptionName);
        }

        private void CreateSubscription()
        {
            var namespaceManager = NamespaceManager.Create();

            // Look for topic
            if (!namespaceManager.TopicExists(_topicName))
                return;

            var topic = namespaceManager.GetTopicAsync(_topicName);

            if (!namespaceManager.SubscriptionExists(_topicName, _subscriptionName))
            {
                if (this._sqlFilter != null)
                {
                    var newOrderFilter = new SqlFilter(this._sqlFilter);
                    namespaceManager.CreateSubscription(_topicName, _subscriptionName, newOrderFilter);
                }
                else
                    namespaceManager.CreateSubscription(_topicName, _subscriptionName);
            }
        }
        public async Task ProcessSubscriptionAsync()
        {
            this.IsRunning = true;

            while (this.IsRunning)
            {
                bool delay = false;

                try
                {
                    // Get a message. The serverWaitTime parameter will give us a delay in our processing loop
                    // when there are no messages
                    var message = await this._subscriptionClient.ReceiveAsync(TimeSpan.FromSeconds(5));

                    // Parse message
                    if (message != null)
                    {
                        await this.ProcessMessage(message);
                    }

                }
                catch (MessagingException e)
                {
                    if (!e.IsTransient)
                        throw;
                    else
                        delay = true;
                }

                // Wait for transient fault to clear
                if (delay)
                    await Task.Delay(2000);
            }
        }

        public void Stop()
        {
            this.IsRunning = false;
        }

        protected abstract Task ProcessMessage(BrokeredMessage message);

        public void Dispose()
        {
            if (this._subscriptionClient != null)
                this._subscriptionClient.Close();
        }
    }
}
