﻿using System;
using System.Diagnostics.Tracing;

namespace AzureBakery.Sales.CustomerWebsite.Instrumentation
{
    [EventSource(Name = "CustomerWebsiteEvents")]
    public class CustomerWebsiteEvents : EventSource
    {
        public static class Keywords
        {
            public const EventKeywords Application = (EventKeywords)1L;
            public const EventKeywords DataAccess = (EventKeywords)2L;
            public const EventKeywords Controller = (EventKeywords)4L;
        }

        public static class Tasks
        {
            // For LogActionFilter
            public const EventTask Action = (EventTask)1;
            public const EventTask Result = (EventTask)2;

            // General
            public const EventTask Initialize = (EventTask)6;
            public const EventTask GetProducts = (EventTask)7;
            public const EventTask AddProductToOrder = (EventTask)8;
        }

        public static class Opcodes
        {
            public const EventOpcode Starting = (EventOpcode)20;
            public const EventOpcode Started = (EventOpcode)21;
            public const EventOpcode Error = (EventOpcode)22;

            public const EventOpcode Ending = (EventOpcode)23;
            public const EventOpcode Ended = (EventOpcode)24;
            
            public const EventOpcode Executed = (EventOpcode)30;
            public const EventOpcode Executing = (EventOpcode)31;
        }

        public static readonly CustomerWebsiteEvents Log = new CustomerWebsiteEvents();

        #region Application

        [Event(100, Level = EventLevel.Verbose, Keywords = Keywords.Application, Task = Tasks.Initialize, Opcode = Opcodes.Starting, Version = 1)]
        public void ApplicationStarting()
        {
            if (this.IsEnabled(EventLevel.Verbose, Keywords.Application))
            {
                this.WriteEvent(100);
            }
        }

        [Event(101, Level = EventLevel.Informational, Keywords = Keywords.Application, Task = Tasks.Initialize, Opcode = Opcodes.Started, Version = 1)]
        public void ApplicationStarted()
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Application))
            {
                this.WriteEvent(101);
            }
        }

        [Event(102, Level = EventLevel.Error, Keywords = Keywords.Application, Opcode = Opcodes.Error, Version = 1)]
        public void ApplicationError(string exceptionMessage, string exceptionType)
        {
            if (this.IsEnabled(EventLevel.Error, Keywords.Application))
            {
                this.WriteEvent(102, exceptionMessage, exceptionType);
            }
        }

        #endregion

        #region LogActionFilter

        [Event(200, Level = EventLevel.Informational, Keywords = Keywords.Controller, Task = Tasks.Action, Opcode = Opcodes.Executing, Version = 1)]
        public void ActionExecuting(string controller, string action)
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Controller))
            {
                this.WriteEvent(200, controller, action);
            }
        }

        [Event(201, Level = EventLevel.Verbose, Keywords = Keywords.Controller, Task = Tasks.Action, Opcode = Opcodes.Executed, Version = 1)]
        public void ActionExecuted(string controller, string action)
        {
            if (this.IsEnabled(EventLevel.Verbose, Keywords.Controller))
            {
                this.WriteEvent(201, controller, action);
            }
        }

        [Event(202, Level = EventLevel.Informational, Keywords = Keywords.Controller, Task = Tasks.Result, Opcode = Opcodes.Executing, Version = 1)]
        public void ResultExecuting(string controller, string action)
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Controller))
            {
                this.WriteEvent(202, controller, action);
            }
        }

        [Event(203, Level = EventLevel.Verbose, Keywords = Keywords.Controller, Task = Tasks.Result, Opcode = Opcodes.Executed, Version = 1)]
        public void ResultExecuted(string controller, string action)
        {
            if (this.IsEnabled(EventLevel.Verbose, Keywords.Controller))
            {
                this.WriteEvent(203, controller, action);
            }
        }        

        #endregion

        #region Product

        [Event(300, Level = EventLevel.Informational, Keywords = Keywords.Controller, Task = Tasks.GetProducts, Opcode = Opcodes.Started, Version = 1)]
        public void ProductControllerGetProductsStarted()
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Controller))
            {
                this.WriteEvent(300);
            }
        }

        [Event(301, Level = EventLevel.Informational, Keywords = Keywords.Controller, Task = Tasks.GetProducts, Opcode = Opcodes.Ended, Version = 1)]
        public void ProductControllerGetProductsEnded()
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Controller))
            {
                this.WriteEvent(301);
            }
        }

        [Event(302, Level = EventLevel.Error, Keywords = Keywords.Controller, Task = Tasks.GetProducts, Opcode = Opcodes.Error, Version = 1)]
        public void ProductControllerGetProductsError(string exceptionMessage, string exceptionType)
        {
            if (this.IsEnabled(EventLevel.Error, Keywords.Controller))
            {
                this.WriteEvent(302, exceptionMessage, exceptionType);
            }
        }

        [Event(303, Level = EventLevel.Informational, Keywords = Keywords.Controller, Task = Tasks.AddProductToOrder, Opcode = Opcodes.Started, Version = 1)]
        public void ProductControllerAddProductToOrderStarted()
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Controller))
            {
                this.WriteEvent(303);
            }
        }

        [Event(304, Level = EventLevel.Informational, Keywords = Keywords.Controller, Task = Tasks.AddProductToOrder, Opcode = Opcodes.Ended, Version = 1)]
        public void ProductControllerAddProductToOrderEnded()
        {
            if (this.IsEnabled(EventLevel.Informational, Keywords.Controller))
            {
                this.WriteEvent(304);
            }
        }

        [Event(305, Level = EventLevel.Error, Keywords = Keywords.Controller, Task = Tasks.AddProductToOrder, Opcode = Opcodes.Error, Version = 1)]
        public void ProductControllerAddProductToOrderError(string exceptionMessage, string exceptionType)
        {
            if (this.IsEnabled(EventLevel.Error, Keywords.Controller))
            {
                this.WriteEvent(305, exceptionMessage, exceptionType);
            }
        }

        #endregion
    }
}