﻿using AzureBakery.Sales.Model;
using AzureBakery.Sales.OrderSim.Services.Intefaces;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace AzureBakery.Sales.OrderSim.Services
{
    public class MessagingService : IMessagingService
    {
        private readonly TopicClient _topicClient = null;
        private const string _topicName = "Order";
        private const string _subscriptionName = "SalesSimSubscription";

        public MessagingService()
        {
            this._topicClient = TopicClient.Create(_topicName);
        }

        public async Task CreateSubscriptionAsync()
        {
            var namespaceManager = NamespaceManager.Create();

            // Look for topic
            if (! await namespaceManager.TopicExistsAsync(_topicName))
                return;

            var topic = await namespaceManager.GetTopicAsync(_topicName);

            if (!await namespaceManager.SubscriptionExistsAsync(_topicName, _subscriptionName))
            {
                var newOrderFilter = new SqlFilter("Status = 'New'");
                await namespaceManager.CreateSubscriptionAsync(_topicName, _subscriptionName, newOrderFilter);
            }
        }

        public async Task<int> PurgeSubscriptionAsync()
        {
            // PeekLock is default ReceiveMode which allows us to remove message once successfully processed
            // We don't care about this as we're purging so we'll use ReceiveAndDelete toremove the message straight away
            var agentSubscriptionClient = SubscriptionClient.Create(_topicName, _subscriptionName, ReceiveMode.ReceiveAndDelete);
            int count = 0;

            while (true)
            {
                bool backOff = false;

                try
                {
                    // Get a batch of 100 messages
                    var messages = await agentSubscriptionClient.ReceiveBatchAsync(100, TimeSpan.FromSeconds(5));

                    // If there are no messages, we're done
                    if (messages.Count() == 0)
                        break;

                    count += messages.Count();
                }
                catch (MessagingException e)
                {
                    if (!e.IsTransient)
                        throw;
                    else
                        backOff = true;
                }

                // Wait for transient fault to clear
                if (backOff)
                    await Task.Delay(2000);
            }

            agentSubscriptionClient.Close();

            return count;
        }

        public async Task SendOrderAsync(Order order, int retries = 0)
        {
            var message = new BrokeredMessage(order);
            message.Properties["Status"] = "New";
            message.MessageId = Guid.NewGuid().ToString();

            await this.SendMessageAsync(message, retries);
        }

        private async Task SendMessageAsync(BrokeredMessage message, int retries = 0)
        {
            int retry = 0;

            while (true)
            {
                try
                {
                    await this._topicClient.SendAsync(message);
                    break;
                }
                catch (MessagingException e)
                {
                    if (!e.IsTransient)
                        throw;
                    else
                        retry++;                        
                    }

                if (retry > retries)
                    break;

                // Back-off for retry
                await Task.Delay(2000);
            } 
        }
    }
}
