﻿using Microsoft.Azure.ActiveDirectory.GraphClient;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Security.Claims;
using System.Web;
using System.Web.Mvc;

namespace AdminWebsite.Auth
{
    [AttributeUsageAttribute(AttributeTargets.Class | AttributeTargets.Method, Inherited = true, AllowMultiple = true)]
    public class AzureAdAuthorizeAttribute : AuthorizeAttribute
    {
        private readonly string _clientId = null;
        private readonly string _appKey = null;
        private readonly string _graphResourceID = "https://graph.windows.net";


        public string AdGroup { get; set; }
        public string AdGroupObjectId { get; set; }

        public AzureAdAuthorizeAttribute()
        {
            this._clientId = ConfigurationManager.AppSettings["ida:ClientID"];
            this._appKey = ConfigurationManager.AppSettings["ida:Password"];
        }
        
        protected override bool AuthorizeCore(HttpContextBase httpContext)
        {
            // First check if user is authenticated
            if (!ClaimsPrincipal.Current.Identity.IsAuthenticated)
                return false;
            else if (this.AdGroup == null && this.AdGroupObjectId == null) // If there are no groups return here
                return base.AuthorizeCore(httpContext);
            
            // Now check if user is in group by querying Azure AD Graph API using client
            bool inGroup = false;
            
            try
            {
                // Get information from user claim
                string signedInUserId = ClaimsPrincipal.Current.FindFirst(ClaimTypes.NameIdentifier).Value;
                string tenantId = ClaimsPrincipal.Current.FindFirst("http://schemas.microsoft.com/identity/claims/tenantid").Value;
                string userObjectId = ClaimsPrincipal.Current.FindFirst("http://schemas.microsoft.com/identity/claims/objectidentifier").Value;


                // Get AuthenticationResult for access token
                var clientCred = new ClientCredential(_clientId, _appKey);
                var authContext = new AuthenticationContext(string.Format("https://login.windows.net/{0}", tenantId));
                var authResult = authContext.AcquireToken(_graphResourceID, clientCred);
                
                // Create graph connection with our access token and API version
                var currentCallContext = new CallContext(authResult.AccessToken, Guid.NewGuid(), "2013-11-08");
                var graphConnection = new GraphConnection(currentCallContext);

                // If we don't have a group id, we can quiery the graph API to find it
                if (this.AdGroupObjectId == null)
                {
                    // Get all groups
                    var groups = graphConnection.List<Group>(null, null);

                    if (groups != null && groups.Results != null)
                    {
                        // Find group object
                        var group = groups.Results.SingleOrDefault(r => (r as Group).DisplayName == this.AdGroup);


                        // check if user is in group
                        if (group != null)
                            this.AdGroupObjectId = group.ObjectId;
                    }
                }

                if (this.AdGroupObjectId != null)
                    inGroup = graphConnection.IsMemberOf(this.AdGroupObjectId, userObjectId);
            }
            catch(Exception ex)
            {
                string message = string.Format("Unable to authorize AD user: {0} against group: {1}", ClaimsPrincipal.Current.Identity.Name, this.AdGroup);
                
                throw new Exception(message, ex);
            }

            return inGroup;
        }
    }
}